import requests
import json
import time
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from os import listdir
from os.path import isfile, join
import rdflib

def recursiveNavigation(g, objects):
	print("\t"+str(len(objects)))
	count = 0
	for obj in objects:
		#sm_id = str(obj[0])
		#request_url = url+"?query="+str(requests.utils.quote("DESCRIBE <"+str(sm_id)+">"))
		#g.parse(request_url)
		count += 1
		global total_requests
		total_requests +=1
		if count > 30:
			break
	print("stoped with :"+str(count))

def runM2MQuery(queryM2M, queryW3C, limit):
	start_time = time.time()
	jresponse = runQuery(queryM2M) # get semantic descriptors ids
	g = rdflib.Graph()
	global total_requests
	total_requests +=1
	# For each id retrieve semantic descriptor
	for result in jresponse["results"]["bindings"]:
		sm_id = result["v0"]["value"]
		request_url = url+"?query="+str(requests.utils.quote("DESCRIBE <"+str(sm_id)+">"))
		total_requests +=1
		g.parse(request_url)
	# solve actual query
	query_W3C_result = g.query(queryW3C)
	if len(query_W3C_result)!=limit:
		# Recursive navigation
		objs = g.query("SELECT ?o WHERE { ?s ?p ?o . FILTER(regex(str(?o), \"^http://stf-589-sim.linkeddata.es/syntectic-\" ) )}")
		recursiveNavigation(g, objs)
		query_W3C_result = g.query(queryW3C)
		if len(query_W3C_result)!=limit:
			print("WARNING M2M query")
	response_time = (time.time() - start_time)
	return round(response_time,2)

def runW3CQuery(query):
	start_time = time.time()
	runQuery(query)
	response_time = (time.time() - start_time)
	return round(response_time,2)

def runQuery(query):
	response = requests.get(url,params={'query': query}, headers = {"Accept": "application/sparql-results+json"})
	if response.status_code != 200:
		print("WARNING! "+query)
	return json.loads(response.text)

def warmUp():
	for index in range(0,1):
		query2 = "SELECT * WHERE { ?s a <https://saref.etsi.org/core/Device>}"
		runW3CQuery(query2)

def limitQuery(query, index):
	return query+" LIMIT "+str(index)

def readQuery(file):
	f = open(file, "r")
	return f.read()

url = 'https://stf-589-sim.linkeddata.es/repositories/stf'
url = 'http://localhost:7200/repositories/discovery'
queries_folder1 = "./queries-W3C/"
queries_folder2 = "./queries-oneM2M/"
total_requests = 0 

warmUp()
limits = np.array([1, 50, 100, 150, 200, 250, 300, 350, 400, 450, 500])
query_count = np.array(list(range(1,13)))
results = pd.DataFrame({})
results["limits"] = limits
for query_num in query_count: # queries
	query_name = str(query_num)+".txt"
	number_requests = []
	for limit in limits: # limits
		total_requests = 0
		query_W3C = limitQuery(readQuery(queries_folder1+query_name), limit)
		query_M2M = limitQuery(readQuery(queries_folder2+query_name), limit)
		runM2MQuery(query_M2M,query_W3C, limit)
		number_requests.append(total_requests)
		print("Limit: "+str(limit)+", query: "+str(query_num)+" requests: "+str(total_requests))
	print("\tsolved query: "+str(query_num))
	print(".--------------------------")
	results["W3C-"+str(query_num)] = number_requests
	#results["M2M-"+str(query_num)] = response_times_M2M
	#results.to_csv('./stf-results-'+str(query_num)+'.csv', index = True)

# Plot
results.to_csv('./stf-response-results.csv', index = True)


